#!/usr/bin/perl

#
# sample script to list open OX sessions via Jolokia
#

# Author: Open-Xchange GmbH, Dominik Epple, dominik.epple@open-xchange.com
# 2015-12-29

# Prerequisites:
# + have Jolokia enabled in your OX config
# + put Jolokia user / password below

use strict;
use warnings;

use LWP::UserAgent;
use LWP::ConnCache;
use JSON;

use MyParser;

my $user="ox";
my $password="secret";
my $baseurl = "http://$user:$password\@127.0.0.1:8009/monitoring/jolokia";

# default is "sessions-6". Only need to change that if you touched com.openexchange.hazelcast.configuration.map.name=sessions-6
# in /opt/open-xchange/etc/hazelcast/sessions.properties
my $sessions_name="sessions-6";

my $ua       = LWP::UserAgent->new();

my $response;
my $content_text;
my $content_json;
my $url;
my $sessions_map;
my $debug=0;

$url = URI->new($baseurl."/search/com.hazelcast:instance=*,name=$sessions_name,type=IMap");
$response = $ua->get($url);
unless ($response->is_success) {
    die $response->status_line;
}
$content_text  = $response->decoded_content();
$content_json = decode_json $content_text;

my $value = ${$content_json}{"value"};
for my $v (@$value) {
    #print "$v\n";
    # actually I dont know whether / in which circumstances more than one value can be retured here...
    if($v =~ /$sessions_name/) {
        $sessions_map = $v;
    }
}

die "unable to read sessions_map from jolokia" unless($sessions_map);
print "sessions_map: $sessions_map\n" if $debug;

$url = URI->new($baseurl."/exec/$sessions_map/entrySet/\"\"");
$response = $ua->get($url);
unless ($response->is_success) {
    die $response->status_line;
}
$content_text  = $response->decoded_content();
$content_json = decode_json $content_text;
print "content_text: ",$content_text,"\n" if $debug;

die "got some error: ". $content_json->{"error"} if $content_json->{"error"};

my $value_str = $content_json->{"value"};
print "value_str: ",$value_str,"\n" if $debug;

if($value_str eq "Empty") { exit 0; }

# unfortunately this is no valid json
# [{key:dce36291f7944edca8d07004ecb67b85, value:{loginName=testuser, password=*****, contextId=1, userId=3, sessionId=dce36291f7944edca8d07004ecb67b85, secret=429d4946303d4418a7d7879fdad1bb60, login=testuser@localdomain, randomToken=983f5a9b377f4411b2d683d9b89cb760, localIp=192.168.1.131, authId=00e0af95d6be41968ffb579c5de33f6c, hash=m7Qrh3BbQUe4VhriMJKjiQ, client=open-xchange-appsuite, userLogin=testuser, parameters={__session.altId=8eb1da5f4c214a598ca694eaaa15f37d}}},]
# what the f...
# need custom parser for that crap -> moved in other module to not spoil the brevity of this script

my ($result, $rest) = MyParser::parse_node($value_str);

#my $result_str = encode_json($result);
#print $result_str,"\n";

my @fields = qw( contextId userId login userLogin localIp );

for my $e (@$result) {
    my $h = $e->{"value"};
    my @values = map { "$_:$h->{$_}" } @fields;
    print join(" ", @values), "\n";
}

__END__
