package MyParser;

#
# parser to parse some json-similar, but not exactly json like answer for the listsessions.pl script
# proof of concept quality. Use on your own risk in production / do some testing and hardening!
# Feedback / improvements welcome to the auther (see below)
#

# Author: Open-Xchange GmbH, Dominik Epple, dominik.epple@open-xchange.com
# 2015-12-29

# this code should parse a response like
# [{key:ff0fbbcf1a834d248c90d8066661d138, value:{loginName=otheruser, password=*****, contextId=1, userId=4, sessionId=ff0fbbcf1a834d248c90d8066661d138, secret=631fa4b7761f441aa72b4457823ab654, login=otheruser@localdomain, randomToken=d123a2d95bb646d090cc535f73589f08, localIp=192.168.1.131, authId=ee2bbd4699ef40128f9a76fa16419391, hash=eBTDW84KTnWlrJzIIQTQ2w, client=open-xchange-appsuite, userLogin=otheruser, parameters={__session.altId=220af1d65dd444d58f3cfcbe7296643e}}}, {key:28bed72869a64fc9b281f89d13fc5bdb, value:{loginName=testuser, password=*****, contextId=1, userId=3, sessionId=28bed72869a64fc9b281f89d13fc5bdb, secret=74f6807b9064446a8d903e83bd32b436, login=testuser@localdomain, randomToken=0ce6a5bb328948e8a31f14e27e36684e, localIp=192.168.1.131, authId=d37d999da04a402abf34b37fac164a43, hash=m7Qrh3BbQUe4VhriMJKjiQ, client=open-xchange-appsuite, userLogin=testuser, parameters={__session.altId=b917e40fe9b44db0b2cccdc4808e26c0}}},]

use strict;
use warnings;

sub peek {
    my ($thing, $expect) = @_;
    die "expect undefined at peek at: \"$thing\"\n" unless($expect);
    if(substr($thing, 0, length $expect) eq $expect) { return 1; }
    return 0;
}

sub consume {
    my $thing=shift(@_);
    my $expect=shift(@_);
    if(peek($thing, $expect)) { return substr($thing, length $expect); }
    die "parse error: expected \"$expect\", thing is: \"$thing\"";
}

sub get_bareword {
    my $thing = shift(@_);
    if($thing =~ /^([a-zA-Z_][a-zA-Z0-9_\.]*)(.*)/) {
        return ($1, $2);
    }
    die "unable to get bareword from thing: $thing\n";
}

sub parse_key_value {
    my $thing = shift(@_);
    my $key;
    ($key, $thing) = get_bareword($thing);
    if(peek($thing, ":")) {
        $thing = consume($thing, ":");
    }
    elsif(peek($thing, "=")) {
        $thing = consume($thing, "=");
    }
    else { die "unable to parse key/value pair: no valid key/value separator found: $thing\n"; }
    my $value;
    ($value, $thing) = parse_node($thing);
    return ($key, $value, $thing);
}

sub parse_dict {
    my $thing = shift(@_);
    my %dict;
    while(1) {
        my ($key, $value);
        ($key, $value, $thing) = parse_key_value($thing);
        $dict{$key}=$value;
        while(peek($thing, " ")) { $thing = consume($thing, " "); }
        if(peek($thing, "}")) {
            return (\%dict, $thing);
        }
        $thing = consume($thing, ",");
        while(peek($thing, " ")) { $thing = consume($thing, " "); }
        if(peek($thing, "}")) {
            return (\%dict, $thing);
        }
    }
}

sub parse_list {
    my $thing = shift(@_);
    my @array;
    while(1) {
        my $element;
        ($element, $thing) = parse_node($thing);
        push @array, $element;
        while(peek($thing, " ")) { $thing = consume($thing, " "); }
        if(peek($thing, "]")) {
            return (\@array, $thing);
        }
        $thing = consume($thing, ",");
        while(peek($thing, " ")) { $thing = consume($thing, " "); }
        if(peek($thing, "]")) {
            return (\@array, $thing);
        }
    }
}

sub parse_node {
    my $thing = shift(@_);
    if(peek($thing, "[")) {
        $thing = consume($thing, "[");
        my $result;
        ($result, $thing) = parse_list($thing);
        $thing = consume($thing, "]");
        return ($result, $thing);
    }
    elsif(peek($thing, "{")) {
        $thing = consume($thing, "{");
        my $result;
        ($result, $thing) = parse_dict($thing);
        $thing = consume($thing, "}");
        return ($result, $thing);
    }
    else {
        if($thing =~ /^([a-zA-Z0-9_\*@\.\-]*)(.*)/) {
            return ($1, $2);
        }
        else {
            die "unable to parse value node: $thing\n";
        }
    }
    die "I should not be here: node matches neither list, nor dict, nor scalar: \"$thing\"\n";
}

1;
